<?php
/**
 * Main application class
 *
 * @package wp-review-me
 * @author InstaProduct
 * @link http://insta-product.com/
 */

//
//Include all function files
require_once( IPREV_LIB . '/config.php' );
require_once( IPREV_LIB . '/setting.php' );
require_once( IPREV_LIB . '/font.php' );
require_once( IPREV_LIB . '/faicon.php' );
require_once( IPREV_LIB . '/modelPage.php' );
require_once( IPREV_LIB . '/modelStyle.php' );
require_once( IPREV_APP . '/controller.php' );
require_once( IPREV_APP . '/viewer.php' );

if ( ! class_exists( 'IPRev_App' ))
{
  class IPRev_App
  {
    //{{ run

    /**
     * Run all functions
     *
     * @return void
     */
    public static function run()
    {
      $hook = sanitize_title( IPREV_TITLE );

      //
      //Admin init
      add_action( 'admin_init', array( __CLASS__, 'adminInit' ) );

      //Install Plugin
      register_activation_hook( __FILE__, array( __CLASS__, 'install' ) );

      //App Init
     add_action( 'init', array( __CLASS__, 'appInit' ) );

      //
      //Register admin menu
      add_action( 'admin_menu', array( __CLASS__, 'addAdminMenu' ) );

      //
      //Review settings
      add_filter( 'admin_comment_types_dropdown', array( __CLASS__, 'addCommentType' ) );
      add_filter( 'comment_row_actions', array( __CLASS__, 'addCommentActions' ), 10, 2 );
      add_filter( 'get_edit_post_link', array( __CLASS__, 'getReviewEditLink' ), 10, 3 );
      add_filter( 'post_type_link', array( __CLASS__, 'getReviewViewLink' ), 10, 3 );
      add_action( 'add_meta_boxes_comment', array( __CLASS__, 'addMetaBoxComment' ) );
      add_action( 'edit_comment', array( __CLASS__, 'saveAdditionalComment' ) );


      /*
       * Rewrite Handler
       */
      add_action( 'pre_get_posts', array( __CLASS__, 'rewriteInit' ) );

      //Render page
      add_action( 'template_redirect', array( __CLASS__, 'renderReviewPage' ) );

      //
      //Add css
      add_action( 'admin_print_styles-toplevel_page_' . IPREV_PAGENAME, array( __CLASS__, 'adminStyle' ), 1100 );
      add_action( 'admin_print_styles-'.$hook.'_page_' . IPREV_PAGENAME . '-style', array( __CLASS__, 'adminStyle' ), 1100 );
      add_action( 'admin_print_styles-'.$hook.'_page_' . IPREV_PAGENAME . '-page', array( __CLASS__, 'adminStyle' ), 1100 );
      add_action( 'admin_print_styles-'.$hook.'_page_' . IPREV_PAGENAME . '-setting', array( __CLASS__, 'adminStyle' ), 1100 );
      add_action( 'admin_print_styles', array( __CLASS__, 'adminGlobalStyle' ), 1000 );

      //
      //Add javascript
      add_action( 'admin_print_scripts-toplevel_page_' . IPREV_PAGENAME, array( __CLASS__, 'adminScript' ), 1100 );
      add_action( 'admin_print_scripts-'.$hook.'_page_' . IPREV_PAGENAME . '-style', array( __CLASS__, 'adminScript' ), 1100 );
      add_action( 'admin_print_scripts-'.$hook.'_page_' . IPREV_PAGENAME . '-page', array( __CLASS__, 'adminScript' ), 1100 );
      add_action( 'admin_print_scripts-'.$hook.'_page_' . IPREV_PAGENAME . '-setting', array( __CLASS__, 'adminScript' ), 1100 );
      add_action( 'admin_print_scripts', array( __CLASS__, 'adminGlobalScript' ), 1000 );


      //
      //Shortcode setup
      //

      // Add generator shortcode button
      add_action( 'init', array( __CLASS__, 'addShortcodeEditorButton' ), 100 );
      // Add pop up content
      add_filter( 'the_editor', array( __CLASS__, 'shortcodeEditorFilter' ) );
      add_action( 'admin_footer', array( __CLASS__, 'addShortcodeForm' ) );
      //Add Shortcode
      add_shortcode( 'review_plus', array( __CLASS__, 'shortcode' ) );

      /*
       * Add meta box option
       */
      add_action( 'add_meta_boxes', array( __CLASS__, 'addMetaBox' ) );
      add_action( 'save_post', array( __CLASS__, 'savePost' ) );

      //Add combo bar via meta box
      add_action( 'wp_footer', array( __CLASS__, 'comboBar' ) );

      /*
       * Register Widget
       */
      add_action( 'widgets_init', array( __CLASS__, 'widgetInit' ));

      //
      //Front style & script
      add_action('wp_head', array( __CLASS__, 'publicStyle' ) );

      if ( ! is_admin() ) {
        add_action('wp_enqueue_scripts', array( __CLASS__, 'publicScript' ), 10 );
      }
    }

    //}}
    //{{ install

    /**
     * Install plugin
     *
     * @return void
     */
    public static function install()
    {
        //Rewrite
        self::registerRewrite();
        flush_rewrite_rules();
    }

    //}}
    //{{ appInit

    /**
     * App init
     *
     * @return void
     */
    public static function appInit()
    {
        //Rewrite
        self::rewriteFlush();

        //Register post type
        $args = array(
      		'show_ui'              => false,
      		'show_in_menu'         => false,
      		'show_in_nav_menus'    => false,
      		'show_in_admin_bar'    => false,
          'labels'  => array(
            'view_item' => __( 'View review page', IPREV_SLUG )
          )
        );
        register_post_type( 'iprev-page-item', $args );
    }

    //}}
    //{{ addAdminMenu

    /**
     * Add admin menu
     *
     * @return void
     */
    public static function addAdminMenu()
    {
      add_menu_page(
        IPREV_TITLE,
        IPREV_TITLE,
        'edit_posts',
        IPREV_PAGENAME,
        array( __CLASS__, 'dashboard' ),
        'dashicons-star-filled'
      );
      add_submenu_page(
        IPREV_PAGENAME,
        __( 'All Reviews', IPREV_SLUG ),
        __( 'All Reviews', IPREV_SLUG ),
        'edit_posts',
        IPREV_PAGENAME,
        array( __CLASS__, 'dashboard' )
      );
      add_submenu_page(
        IPREV_PAGENAME,
        __( 'Review Page', IPREV_SLUG ),
        __( 'Review Page', IPREV_SLUG ),
        'edit_posts',
        IPREV_PAGENAME . '-page',
        array( __CLASS__, 'page' )
      );
      add_submenu_page(
        IPREV_PAGENAME,
        __( 'Review Style', IPREV_SLUG ),
        __( 'Review Style', IPREV_SLUG ),
        'edit_posts',
        IPREV_PAGENAME . '-style',
        array( __CLASS__, 'style' )
      );
      add_submenu_page(
        IPREV_PAGENAME,
        __( 'Settings', IPREV_SLUG ),
        __( 'Settings', IPREV_SLUG ),
        'edit_posts',
        IPREV_PAGENAME . '-setting',
        array( __CLASS__, 'setting' )
      );
    }

    //}}
    //{{ adminInit

    /**
     * Admin init
     *
     * @return void
     */
    public static function adminInit()
    {
      /**
       * Ajax handler
       **/
      $controller = new IPRev_Controller();
      $controller->ajax();
    }

    //}}
    //{{ dashboard

    /**
     * Render admin dashboard page
     *
     * @return void
     */
    public static function dashboard()
    {
      $viewer = new IPRev_Viewer();
      if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
        $viewer->edit();
      } else {
        $viewer->dashboard();
      }
    }

    //}}
    //{{ page

    /**
     * Render admin dashboard page
     *
     * @return void
     */
    public static function page()
    {
      $viewer = new IPRev_Viewer();
      if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
        $viewer->editPage();
      } elseif ( ! empty( $_GET['action'] ) && $_GET['action'] == 'create' ) {
        $viewer->createPage();
      } else {
        $viewer->page();
      }
    }

    //}}
    //{{ style

    /**
     * Render style page
     *
     * @return void
     */
    public static function style()
    {
      $viewer = new IPRev_Viewer();
      if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
        $viewer->editStyle();
      } elseif ( ! empty( $_GET['action'] ) && $_GET['action'] == 'create' ) {
        $viewer->createStyle();
      } else {
        $viewer->style();
      }
    }

    //}}
    //{{ setting

    /**
     * Render setting page
     *
     * @return void
     */
    public static function setting()
    {
      $viewer = new IPRev_Viewer();
      $viewer->setting();
    }

    //}}
    //{{ rewriteInit

    /**
     * Init request
     *
     * @return void
     */
    public static function rewriteInit( $query )
    {
        $q = $query->query;
        if ( isset( $q['iprev'] ) && isset( $q['review_name'] )) {
            $query->query_vars['post_type'] = 'iprev-page-item';
            $query->query_vars['name']      = $q['review_name'];
            $query->query_vars['pagename']  = $q['review_name'];
            $query->query['pagename']       = $q['review_name'];

        } elseif( isset( $q['review_id'] )) {
            $query->query_vars['post_type'] = 'iprev-page-item';
            $query->query_vars['p']         = absint( $q['review_id'] );
        }
    }

    //}}
    //{{ registerRewrite

    /**
     * Init rewrite rule
     *
     * @return void
     */
    public static function registerRewrite()
    {
        $base = get_option( '__iprev_review_page_base_path', 'review' );
        add_rewrite_rule(
            $base . '/([^/]+)/?$',
            'index.php?iprev=1&review_name=$matches[1]',
            'top'
        );
        add_rewrite_rule(
            $base . '/([^/]+)/feed/(feed|rdf|rss|rss2|atom)/?$',
            'index.php?iprev=1&review_name=$matches[1]&feed=$matches[2]',
            'top'
        );
        add_rewrite_rule(
            $base . '/([^/]+)/(feed|rdf|rss|rss2|atom)/?$',
            'index.php?iprev=1&review_name=$matches[1]&feed=$matches[2]',
            'top'
        );
        add_rewrite_rule(
            $base . '/([^/]+)/page/?([0-9]{1,})/?$',
            'index.php?iprev=1&review_name=$matches[1]&paged=$matches[2]',
            'top'
        );
        add_rewrite_tag( '%review_name%', '([a-zA-Z0-9\_])');
        add_rewrite_tag( '%review_id%', '([0-9])');
        add_rewrite_tag( '%iprev%', '([1])');
    }

    //}}
    //{{ rewriteFlush

    /**
     * Flush rewrite rule
     *
     * @return void
     */
    public static function rewriteFlush()
    {
        $base = get_option( '__iprev_review_page_base_path', 'review' );
        $old  = get_option( '__iprev_review_page_base_path_old', 'review' );

        self::registerRewrite();

        if ( $base != $old ) {
            flush_rewrite_rules();
            update_option( '__iprev_review_page_base_path_old', $base );
        }
    }

    //}}
    //{{ renderReviewPage

    /**
     * Render review page
     *
     * @return void
     */
    public static function renderReviewPage()
    {
        //Render view
        $viewer = new IPRev_Viewer();
        $viewer->reviewPage();
    }

    //}}
    //{{ addShortcodeEditorButton

    /**
     * Add shortcode button in content editor
     *
     * @return void
     */
    public static function addShortcodeEditorButton()
    {
      if ( ! current_user_can('edit_posts') && ! current_user_can('edit_pages') ) {
        return;
      }
      if ( get_user_option('rich_editing') == 'true') {
        add_filter( 'mce_external_plugins', array( __CLASS__, 'addTinymcePlugin' ));
        add_filter( 'mce_buttons', array( __CLASS__, 'registerTinymceButton' ));
      }
    }

    //}}
    //{{ addTinymcePlugin

    /**
     * Add tinyMce plugin
     *
     * @return void
     */
    public static function addTinymcePlugin( $plugin_array )
    {
      $plugin_array['iprev_shortcode'] = IPREV_BASE_URL . '/assets/js/editor-plugin.js';
      return $plugin_array;
    }

    //}}
    //{{ registerTinymceButton

    /**
     * Add tinyMce button
     *
     * @return void
     */
    public static function registerTinymceButton( $buttons )
    {
      array_push($buttons, "|", "iprev_shortcode");
      return $buttons;
    }

    //}}
    //{{ shortcodeEditorFilter

    /**
     * The editor filter
     *
     * @param  string $content
     * @return string
     */
    public static function shortcodeEditorFilter( $content )
    {
      global $iprev_useEditor;
      $iprev_useEditor = true;
      return $content;
    }

    //}}
    //{{ addShortcodeForm

    /**
     * Add shortcode form
     *
     * @return void
     */
    public static function addShortcodeForm()
    {
      global $iprev_useEditor;

      wp_enqueue_script('wp-color-picker');
      wp_enqueue_style('wp-color-picker');

      if ( $iprev_useEditor ) {
        $viewer = new IPRev_Viewer();
        $viewer->shortcodeBackend();
      }
    }

    //}}
    //{{ shortcode

    /**
     * Get shortcode
     *
     * @param  array  $atts
     * @param  string $content
     * @return void
     */
    public static function shortcode( $atts, $content = '')
    {
      $viewer = new IPRev_Viewer();
      return $viewer->shortcode( $atts, $content );
    }

    //}}
    //{{ addMetaBox

    /**
     * Add administrator meta box
     *
     * @return void
     */
    public static function addMetaBox()
    {
        //
        //Add post/page metabox
        $screens = array( 'post', 'page' );
        foreach ($screens as $screen)
        {
            add_meta_box(
                'iprev_metabox',
                __( 'Review Combo Bar Settings', IPREV_SLUG ),
                array( __CLASS__, 'metaBoxContent' ),
                $screen
            );
        }
    }

    //}}
    //{{ addMetaBoxComment

    /**
     * Add administrator meta box
     *
     * @return void
     */
    public static function addMetaBoxComment( $comment )
    {
      if ( $comment->comment_type == 'iprev_item' )
      {
        add_meta_box(
         'iprev-review-meta-box',
         __( 'Additional Information', IPREV_SLUG ),
         array( __CLASS__, 'metaBoxCommentContent' ),
         'comment',
         'normal'
       );
      }
    }

    //}}
    //{{ metaBoxContent

    /**
     * Content for administrator meta box
     *
     * @return void
     */
    public static function metaBoxContent( $post )
    {
        $viewer = new IPRev_Viewer();
        $viewer->metabox( $post );
    }

    //}}
    //{{ metaBoxCommentContent

    /**
     * Content for administrator meta box
     *
     * @return void
     */
    public static function metaBoxCommentContent( $comment )
    {
        $viewer = new IPRev_Viewer();
        $viewer->metaBoxComment( $comment );
    }

    //}}
    //{{ savePost

    /**
     * Save page/post meta box setting
     *
     * @return void
     */
    public static function savePost( $post_id )
    {
        if ( isset( $_POST['post_type'] ) && 'page' == $_POST['post_type'] ) {
            if ( ! current_user_can( 'edit_page', $post_id ) ) {
                return;
            }
        } else {
            if ( ! current_user_can( 'edit_post', $post_id ) ) {
                return;
            }
        }
        // Check if the user intended to change this value.
        if ( ! isset( $_POST['iprev_mbox_nonce'] ) ) {
            return;
        }
        $post_ID  = $_POST['post_ID'];
        //Review Page
        $pageId = sanitize_text_field( $_POST['iprev_review_page'] );
        update_post_meta( $post_ID, '__iprev_page_review_page', $pageId );
        //Style
        $style = sanitize_text_field( $_POST['iprev_style'] );
        update_post_meta( $post_ID, '__iprev_page_style', $style );
    }

    //}}
    //{{ saveAdditionalComment

    /**
     * Save Additional comment data
     *
     * @return void
     */
    public static function saveAdditionalComment( $commentID )
    {
      global $wpdb;

      // check that comment ID exists
      $comment = get_comment( $commentID );
      if ( ! $comment ) {
        return true; // don't update
      }

      // Update data
      //
      if ( $comment->comment_type == 'iprev_item' )
      {
        //Title
        if ( isset( $_POST['review_title'] )) {
          $title = apply_filters( 'single_post_title', $_POST['review_title'] );
          update_comment_meta( $commentID, 'review_title', $title );
        }
        //
        //Job
        if ( isset( $_POST['review_user_job'] )) {
          $job = apply_filters( 'single_post_title', $_POST['review_user_job'] );
          update_comment_meta( $commentID, 'review_user_job', $job );
        }
        //
        //Rating
        if ( isset( $_POST['review_rating'] ))
        {
          $rating = absint( $_POST['review_rating'] );
          update_comment_meta( $commentID, 'review_rating', $rating );

          $data = array(
            'comment_karma' => $rating
          );
          $commentarr = array(
            'comment_ID' => $commentID
          );
          $rval = $wpdb->update( $wpdb->comments, $data, $commentarr );
        }
      }

      return true;
    }

    //}}
    //{{ comboBar

    /**
     * Get combo bar backend page
     *
     * @return void
     */
    public static function comboBar()
    {
        $viewer = new IPRev_Viewer();
        return $viewer->comboBar();
    }

    //}}
    //{{ widgetInit

    /**
     * Widget innit
     */
    public static function widgetInit()
    {
        require_once( IPREV_LIB . 'widget.php' );
        register_widget( 'IPRev_Widget' );
    }

    //}}
    //{{ addCommentType

    /**
     * Add comment type
     */
    public static function addCommentType( $options )
    {
      $options['iprev_item'] = __('Review', IPREV_SLUG);
      return $options;
    }

    //}}
    //{{ addCommentActions

    /**
     * Add comment actions
     */
    public static function addCommentActions( $actions, $comment )
    {
      if ( $comment->comment_type == 'iprev_item' ) {
        if ( isset( $actions['reply'] )) {
          unset( $actions['reply'] );
        }
      }
      return $actions;
    }

    //}}
    //{{ getReviewEditLink

    /**
     * Get review edit link
     */
    public static function getReviewEditLink( $link, $postId, $context )
    {
      if ( ! $post = get_post( $postId )) {
        return $link;
      }
      if ( $post->post_type == 'iprev-page-item' ) {
        if ( $context == 'display' ) {
          $link = admin_url() . 'admin.php?page=' . IPREV_PAGENAME .  '-page&amp;action=edit&amp;id=' . $post->ID;
        } else {
          $link = admin_url() . 'admin.php?page=' . IPREV_PAGENAME .  '-page&action=edit&id=' . $post->ID;
        }
      }
      return $link;
    }

    //}}
    //{{ getReviewViewLink

    /**
     * Get review edit link
     */
    public static function getReviewViewLink( $permalink, $post, $leavename )
    {
      if ( $post->post_type == 'iprev-page-item' )
      {
        $home = get_home_url();
        $path = get_option( '__iprev_review_page_base_path', 'review' );
        $root = $home . '/' . $path . '/';
        $permalink = $root . $post->post_name;
      }
      return $permalink;
    }

    //}}
    //{{ adminStyle

    /**
     * Add Css file
     *
     * @return void
     */
    public static function adminStyle()
    {
      wp_deregister_style( 'iprev-admin-style' );
      wp_register_style('iprev-admin-style', IPREV_BASE_URL . 'assets/css/admin.min.css', false, null);
      wp_enqueue_style('iprev-admin-style');

      wp_deregister_style( 'iprev-datetime-style' );
      wp_register_style('iprev-datetime-style', IPREV_BASE_URL . 'assets/css/jquery.datetimepicker.css', false, null);
      wp_enqueue_style('iprev-datetime-style');
    }

    //}}
    //{{ adminGlobalStyle

    /**
     * Add Css file to all pages
     *
     * @return void
     */
    public static function adminGlobalStyle()
    {
      wp_deregister_style( 'iprev-admin-global-style' );
      wp_register_style('iprev-admin-global-style', IPREV_BASE_URL . 'assets/css/admin-global.min.css', false, null);
      wp_enqueue_style('iprev-admin-global-style');
    }

    //}}
    //{{ adminScript

    /**
     * Add javascript file
     *
     * @return void
     */
    public static function adminScript()
    {
    ?>
      <script type="text/javascript">
        var iprev_img_url = '<?php echo IPREV_BASE_URL . 'assets/img/'; ?>',
          ui_base_image  = '<?php echo IPREV_BASE_URL . 'assets/img/'; ?>',
          ui_message     =
          {
            delete_confirm_title: '<?php _e( 'Delete Confirmation', IPREV_SLUG ); ?>',
            delete_confirm_info: '<?php _e( 'Are you sure doing this action?', IPREV_SLUG ); ?>'
          }
      </script>
      <?php
      wp_deregister_script('iprev-admin-script');
      wp_enqueue_script(
        'iprev-admin-script',
        IPREV_BASE_URL . 'assets/js/admin.js',
        array( 'jquery' ),
        1,
        true
      );
    }

    //}}
    //{{ adminGlobalScript

    /**
     * Add javascript file to all pages
     *
     * @return void
     */
    public static function adminGlobalScript()
    {
      wp_deregister_script('iprev-admin-global-script');
      wp_enqueue_script(
        'iprev-admin-global-script',
        IPREV_BASE_URL . 'assets/js/admin-global.min.js',
        array( 'jquery' ),
        1,
        true
      );
    }

    //}}
    //{{ publicStyle

    /**
     * Print public style
     *
     * @return void
     */
    public static function publicStyle()
    {
    ?>
      <script type="text/javascript">
        var iprev_ajaxurl  = "<?php echo admin_url( 'admin-ajax.php' ); ?>";
        var iprev_img_url  = '<?php echo IPREV_BASE_URL . 'assets/img/'; ?>';
        var iprev_message =
        {
            input_error_name: '<?php _e( 'You must enter your name.', IPREV_SLUG ); ?>',
            input_error_email: '<?php _e( 'You must enter your email address.', IPREV_SLUG ); ?>',
            input_error_email_format: '<?php _e( 'Your email address is wrong format.', IPREV_SLUG ); ?>',
            input_error_content: '<?php _e( 'You must enter your review.', IPREV_SLUG ); ?>',
            submit_success: '<?php _e( 'Review successfully saved.', IPREV_SLUG ); ?>',
            submit_failed: '<?php _e( 'Review failed to save.', IPREV_SLUG ); ?>',
        };
      </script>
      <link media="all" type="text/css" href="<?php echo IPREV_BASE_URL; ?>assets/css/public.min.css" rel="stylesheet">
      <link media="all" type="text/css" href="<?php echo IPREV_BASE_URL; ?>assets/owl-carousel/owl.carousel.css" rel="stylesheet">
      <link media="all" type="text/css" href="<?php echo IPREV_BASE_URL; ?>assets/owl-carousel/owl.theme.css" rel="stylesheet">

      <link href='https://fonts.googleapis.com/css?family=Roboto:400,300italic,300,400italic,700,700italic|Roboto+Condensed:400,700' rel='stylesheet' type='text/css'>

    <?php
    }

    //}}
    //{{ publicScript

    /**
     * Print public script
     *
     * @return void
     */
    public static function publicScript()
    {
      wp_enqueue_script('jquery');

      wp_deregister_script('jquery-ui');
      wp_register_script('jquery-ui', "http" . ($_SERVER['SERVER_PORT'] == 443 ? "s" : "") . "://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/jquery-ui.min.js", false, null);
      wp_enqueue_script('jquery-ui');

      wp_deregister_script('iprev-owl-carousel');
      wp_enqueue_script(
        'iprev-owl-carousel',
        IPREV_BASE_URL . 'assets/owl-carousel/owl.carousel.min.js',
        array( 'jquery' ),
        1,
        true
      );

      wp_deregister_script('iprev-public-script');
      wp_enqueue_script(
        'iprev-public-script',
        IPREV_BASE_URL . 'assets/js/public.min.js',
        array( 'jquery' ),
        1,
        true
      );
    }

    //}}
  }
}
