<?php
/**
 * Controller to handling saving review data
 *
 * @package wp-review-me
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPRev_Ajax_Savereview' ))
{
  class IPRev_Ajax_Savereview
  {
    /**
     * Execution save options process
     *
     * @return void
     */
    public function exec()
    {
      global $user_ID;

      $error = true;
      if ( ! empty( $_POST['page_id'] ))
      {
        $pageId = absint( $_POST['page_id'] );
        if ( $page = get_post( $pageId ))
        {
          //
          //Get user data
          //
          $userdata = array();
          if ( ! empty( $user_ID ))
          {
            $user = get_userdata( $user_ID );
            //Name
            if ( ! empty( $user->display_name )) {
              $name = $user->display_name;
            }
            elseif ( ! empty( $user->user_nickname )) {
              $name = $user->user_nickname;
            }
            else {
              $name = $user->user_login;
            }
            //Email
            $email = $user->user_email;
            //URL
            $url   = $user->user_url;
          }
          else
          {
            $name  = $_POST['name'];
            $email = $_POST['email'];
            $url   = $_POST['url'];
          }
          $userdata = array(
            'name'  => $name,
            'email' => $email,
            'url'   => $url
          );

          $setting = new IPRev_Setting( $pageId, '__iprev-page-setting' );
          $result  = $this->_saveReview( $pageId, $userdata );
          if ( ! empty( $result ) )
          {
            $message = $setting->getSetting( 'message_thank', 'Success' );
            $status  = 1;
            //
            //Send email
            $sendto     = $setting->getSetting( 'admin_email' );
            $send_admin = $setting->getSetting( 'send_email' );
            $subject    = __( 'New Review', IPREV_SLUG );
            $body       =
            'On ' . date( 'd F Y, h:i:s' ) . ', has received a new review with the following data:
            Name: ' . $userdata['name'] . '
            Email: ' . $userdata['email'] . '
            Review: ' . $_POST['content'] . '
            Administrator
            ';

            add_filter( 'wp_mail_content_type', 'iprev_set_html_content_type' );

            if ( ! empty( $sendto )) {
                @wp_mail( $sendto, $subject, $body );
            }
            if ( ! empty( $send_admin )) {
                $admin_email = get_option( 'admin_email' );
                @wp_mail( $admin_email, $subject, $body );
            }
            remove_filter( 'wp_mail_content_type', 'iprev_set_html_content_type' );
            $error = false;
          }
          else {
              $message = $setting->getSetting( 'message_error', 'Error' );
              $status  = -1;
          }
        }
      }
      echo json_encode( array(
        'status'    => $status,
        'message'   => $message
      ) );
      die();
    }

    /**
     * Do save review
     *
     * @return void
     */
    protected function _saveReview( $pageId, $userdata )
    {
      global $user_ID;

      $data = array(
        'comment_post_ID'       => $pageId,
        'comment_type'          => 'iprev_item',
        'comment_parent'        => 0,
        'user_id'               => $user_ID,
        'comment_author_IP'     => $_SERVER['REMOTE_ADDR'],
        'comment_agent'         => $_SERVER['HTTP_USER_AGENT'],
        'comment_date'          => current_time('mysql'),
        'comment_approved'      => 0,
        'comment_author'        => apply_filters( 'single_post_title', $userdata['name'] ),
        'comment_author_email'  => sanitize_email( $userdata['email'] ),
        'comment_author_url'    => '',
        'comment_content'       => $_POST['content'],
      );

      //
      //URL
      if ( isset( $userdata['url'] )) {
        $data['comment_author_url'] = apply_filters( 'pre_link_url', $userdata['url'] );
      }
      //
      //Rating
      if ( isset( $_POST['rating'] )) {
        $data['comment_karma'] = absint( $_POST['rating'] );
      }

      $commentID = wp_insert_comment( $data );
      if ( ! empty( $commentID ))
      {
        //
        //Time
        add_comment_meta( $commentID, 'review_time', time() );
        //
        //Title
        if ( ! empty( $_POST['title'] )) {
          $title = apply_filters( 'single_post_title', $_POST['title'] );
          add_comment_meta( $commentID, 'review_title', $title );
        }
        //
        //Job
        if ( ! empty( $_POST['job'] )) {
          $job = apply_filters( 'single_post_title', $_POST['job'] );
          add_comment_meta( $commentID, 'review_user_job', $job );
        }
        //
        //Rating
        if ( isset( $_POST['rating'] ))
        {
          $rating = absint( $_POST['rating'] );
          add_comment_meta( $commentID, 'review_rating', $rating );

          //
          //Update rating in page
          //
          /*
          $total = get_post_meta( $pageId, '__iprev-page-rating', true );
          if ( empty( $total )) {
            $total = 0;
          }
          $total += $rating;
          update_post_meta( $pageId, '__iprev-page-rating', $total );
          */
        }
      }
      return $commentID;
    }
  }
}
